#!/usr/bin/env python
# coding: utf-8

# MAT 259: Final Project
# 
# Chantal Nguyen
# 
# Instructor: George Legrady
# 
# 
# The following code reads in a json file containing recipes extracted from the online recipe database Epicurious.com. The data includes recipe titles, ratings, review counts, dates of publication, ingredient lists, instructions, etc. 
# 
# tf-idf is used to vectorize the ingredient lists of each recipe. Dimensionality reduction is performed first with SVD and then with t-SNE to project recipes into 2D space. k-means clustering is used to cluster recipes into 15 clusters. 



from sklearn.manifold import TSNE
from sklearn.decomposition import TruncatedSVD
from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.cluster import KMeans
import numpy as np
import io
import re
import json
import pandas as pd
import time
from datetime import datetime


# Load in the json file as a pandas dataframe




# load in the data
data = pd.read_json('epicurious-recipes.json',lines=True)
data = data[data.reviewsCount > 0]
for index, row in data.iterrows(): # clean up entries with no ingredients listed
    if not isinstance(row['ingredients'],(list,)):
        data = data.drop(index)
data = data.reset_index(drop=True)


# Extract ingredient lists and remove numbers and non-alphabetic characters



ingredients_data = []
chars_to_remove = dict((ord(char), None) for char in '/0123456789!"#%\'()*+,-./:;<=>?@[\]^_`{|}~')
for index, row in data.iterrows():
    ingredients_data.append((''.join(row['ingredients'])).translate(chars_to_remove))


# Define stopwords: common English stopwords (found here: https://github.com/stopwords-iso/stopwords-en/) plus words related to measurement quantities



with open('stopwords-en.json') as f:
    stopwords = json.load(f)




stopwords = stopwords + [u'cup', u'cups', u'tablespoon', u'tablespoons', u'teaspoon', u'teaspoons', u'c', u'tbsp', u'tsp', u'oz', u'g', u'kg', u'lb', u'pt', u'gal', u'qt', u'qts', u'tbsps', u'tsps', u'ml', u'l', u'inch', u'inches', u'pinch', u'pinches', u'dash', u'dashes', u'ounce', u'ounces', u'can', u'cans', u'bag', u'bags', u'package', u'packages', u'gram', u'grams', u'pound', u'pounds']


# Vectorize with tf-idf



vectorizer = TfidfVectorizer(max_df=0.5, max_features=10000,
                                 min_df=2, stop_words=stopwords,
                                 use_idf=True)
X = vectorizer.fit_transform(ingredients_data)


# First reduce dimensionality to 50 dimensions with SVD (performing t-SNE now would be too intensive)



svd = TruncatedSVD(n_components=50)
Y = svd.fit_transform(X)


# Reduce to 2D with t-SNE



tsne_model = TSNE(n_components=2, verbose=1, random_state=1)
Z = tsne_model.fit_transform(Y)


# Extract t-SNE coordinates, append to dataframe



x_coords = Z[:,0]
y_coords = Z[:,1]
coords = np.transpose(np.vstack((x_coords, y_coords)))  
with open('epi_coords.csv', 'w') as f:
    for x, y in coords:
        f.write('%f, %f\n' % (x,y)) 




coords_df = pd.DataFrame(np.stack((x_coords,y_coords)).transpose(), columns=list('xy'))




data = data[['hed', 'pubDate', 'aggregateRating','reviewsCount','willMakeAgainPct']]
data = data.join(coords_df)


# Perform k-means clustering into (arbitrarily-set) 15 clusters



num_clusters = 15
km = KMeans(n_clusters=num_clusters, init='k-means++', max_iter=100, n_init=1)
print("Clustering sparse data with %s" % km)
km.fit(X)

print("Top terms per cluster:")
order_centroids = km.cluster_centers_.argsort()[:, ::-1]
terms = vectorizer.get_feature_names()
for i in range(num_clusters):
    print("Cluster %d:" % i)
    for ind in order_centroids[i, :10]:
        print(' %s' % terms[ind])
        




data['label'] = km.labels_


# Do some cleanup and save data as json file containing recipe title, rating, review count, publication date, % of reviewers who said they will make the recipe again, t-SNE coords, and cluster label



datestrings = []
for index, row in data.iterrows():
    pdate = row['pubDate']
    data.at[index,'pubDate'] = time.mktime(datetime.strptime(pdate, '%Y-%m-%dT%H:%M:%S.%fZ').timetuple())/1e+09
    datestrings.append(pdate[0:10])
data['datestr'] = datestrings



with open('epicurious_data.json','w') as f:
    f.write(data.to_json(orient = "records"))







